<?php
/*
	Plugin Name: Keyword Super Box
	Plugin URI: https://sitemile.com
	Description: Provides a keyword box where you can type in your keywords to match them in your post content.
	Tags: keyword, box, seo
	Author: Andrei Saioc
	Author URI: https://sitemile.com
	Version: 1.0.1  
*/

 
add_action('admin_enqueue_scripts', 'load_custom_wp_admin_style_script');

function load_custom_wp_admin_style_script($hook) {
    wp_localize_script('custom-meta-box-script', 'ajax_object',
        array('ajax_url' => admin_url('admin-ajax.php'))
    );
}


// Hook into the 'add_meta_boxes' action
add_action('add_meta_boxes', 'custom_side_meta_box');

// Hook into the 'admin_head' to add custom JS directly in the admin header
add_action('admin_head', 'add_custom_js_to_admin_head');

// Hook for handling AJAX request
add_action('wp_ajax_save_custom_keywords', 'save_custom_keywords_handler');

function custom_side_meta_box() {
    add_meta_box(
        'custom_keywords_meta',
        __('Keyword Super Box', 'textdomain'),
        'custom_keywords_meta_box_content',
       array('post', 'page'),
        'side',
        'default'
    ); 
}

function custom_keywords_meta_box_content($post) {
    wp_nonce_field('custom_keywords_nonce_action', 'custom_keywords_nonce');
    echo '<textarea id="custom_keywords_field" name="custom_keywords_field" rows="4" style="width:100%"></textarea>';
    echo '<button type="button" id="save_custom_keywords" class="button">'.__('Save Keywords', 'textdomain').'</button>';
    echo '<div id="custom_keywords_display" style="display:inline-block; width: 100%; margin-top:14px"></div>';
}



function add_custom_js_to_admin_head() {
    global $pagenow;
    if (!in_array($pagenow, array('post.php', 'post-new.php'))) {
        return;
    }

    $post_id = get_the_ID();
    if ($post_id) {
        $keywords = get_post_meta($post_id, 'custom_keywords', true);
    } else {
        $keywords = '';
    }
    $keywords_js_array = json_encode(explode(',', $keywords));
    ?>
    <script type="text/javascript">
    jQuery(document).ready(function($) {
        var keywords = <?php echo $keywords_js_array; ?>;

        function updateKeywordsDisplay() {

    var editorContent = '';
    // Gutenberg editor
    if (window.wp && wp.data && wp.data.select('core/editor')) {
        console.log("gutenberg ediitor")
        editorContent = wp.data.select('core/editor').getEditedPostContent().toLowerCase(); // Convert to lowercase

        if(editorContent  == "")
        editorContent = jQuery('#content').val().toLowerCase(); // Convert to lowercase

       // console.log(editorContent)

    }
    // Classic Editor - TinyMCE (Visual)
    else if (window.tinyMCE && tinyMCE.activeEditor && !tinyMCE.activeEditor.isHidden()) {

        console.log("classic editor visaul")
        editorContent = tinyMCE.activeEditor.getContent().toLowerCase(); // Convert to lowercase
    }
    // Classic Editor - Textarea (Text)
    else {
        console.log("classic editor textarea")
        editorContent = jQuery('#content').val().toLowerCase(); // Convert to lowercase
    } 



    var $keywordsDisplay = jQuery('#custom_keywords_display');
    $keywordsDisplay.html('');

    keywords.forEach(function(keyword) {
        var keywordLower = keyword.toLowerCase();
        var borderStyle = '1px solid grey';
        if (editorContent.indexOf(keywordLower) !== -1) {
            borderStyle = '2px solid green; font-weight:700; color:green;';
        }
        if (keyword.trim() !== '') {
            $keywordsDisplay.append('<div style="float:left; margin-right: 5px; margin-bottom:5px; border-radius:12px; border: ' + borderStyle + '; padding: 5px; margin-top: 5px;">' + keyword + ' <span class="keyword-delete" data-keyword="' + keyword + '" style="cursor:pointer; color:red;">X</span></div>');
        }
    });
 


   // Attach event listener for deletion
   $('.keyword-delete').click(function() {
        var keywordToDelete = $(this).data('keyword');
        $.post(ajaxurl, {
            action: 'delete_custom_keyword',
            post_id: $('#post_ID').val(),
            keyword: keywordToDelete,
            security: $('#custom_keywords_nonce').val()
        }, function(response) {
            if (response.success) {
                // Update the keywords variable and display
                keywords = response.data;
                updateKeywordsDisplay();
            } else {
                alert('Error: ' + response.data);
            }
        });
    });


}

        // Function to attach the keyup event listener to TinyMCE editors
        function attachTinyMceKeyupListener() {
            if (window.tinyMCE) {
                tinyMCE.editors.forEach(function(editor) {
                    if (!editor._hasKeyupListener) { // Check if the listener is not already attached
                        editor.on('keyup', function() {
                            updateKeywordsDisplay();
                        });
                        editor._hasKeyupListener = true; // Mark this editor as having the listener attached
                    }
                });
            } 
        }

        // Regularly check and attach the keyup event listener to any TinyMCE editors
        setInterval(attachTinyMceKeyupListener, 1000);

        $('#content').on('keyup', function() {
            updateKeywordsDisplay();
        });





        // Update the save keywords click handler to send the keywords with commas
$('#save_custom_keywords').on('click', function() {
    var keywordsStr = $('#custom_keywords_field').val().trim().replace(/[\n\r]+/g, ',');
    $.post(ajaxurl, {
        action: 'save_custom_keywords',
        post_id: $('#post_ID').val(),
        keywords: keywordsStr,
        security: $('#custom_keywords_nonce').val()
    }, function(response) {
        if (response.success) {
            keywords = response.data;
            updateKeywordsDisplay();
            $('#custom_keywords_field').val('');
        } else {
            alert('Error: ' + response.data);
        }
    });
});



        // Event for Gutenberg editor
        if (window.wp && wp.data && wp.data.subscribe) {
            let lastContent = '';
            wp.data.subscribe(function() {
                const editorContent = wp.data.select('core/editor').getEditedPostContent();
                if (lastContent !== editorContent) {
                    lastContent = editorContent;
                    updateKeywordsDisplay();
                }
            });
        }

        // Initial update
        updateKeywordsDisplay();
    });
    </script>
    <?php
}

function save_custom_keywords_handler() {
    check_ajax_referer('custom_keywords_nonce_action', 'security');
    
    $post_id = intval($_POST['post_id']);
    if (!current_user_can('edit_post', $post_id)) {
        wp_send_json_error('Error: insufficient permissions');
        return;
    }

    $new_keywords = sanitize_text_field($_POST['keywords']);
    // Split keywords by comma or new line, then trim each keyword
    $new_keywords_array = preg_split('/[\n\r,]+/', $new_keywords, -1, PREG_SPLIT_NO_EMPTY);
    $new_keywords_array = array_map('trim', $new_keywords_array);

    // Retrieve existing keywords and append new ones
    $existing_keywords = get_post_meta($post_id, 'custom_keywords', true);
    $existing_keywords_array = explode(',', $existing_keywords);
    $updated_keywords_array = array_unique(array_merge($existing_keywords_array, $new_keywords_array));
    $updated_keywords = implode(',', $updated_keywords_array);

    // Save the updated list of keywords
    update_post_meta($post_id, 'custom_keywords', $updated_keywords);

    wp_send_json_success($updated_keywords_array);
}




add_action('wp_ajax_delete_custom_keyword', 'delete_custom_keyword_handler');

function delete_custom_keyword_handler() {
    check_ajax_referer('custom_keywords_nonce_action', 'security');
    
    $post_id = intval($_POST['post_id']);
    if (!current_user_can('edit_post', $post_id)) {
        wp_send_json_error('Error: insufficient permissions');
        return;
    }

    $keyword_to_delete = sanitize_text_field($_POST['keyword']);
    $existing_keywords = get_post_meta($post_id, 'custom_keywords', true);
    $keywords_array = explode(',', $existing_keywords);
    
    // Remove the keyword to delete
    $updated_keywords_array = array_filter($keywords_array, function($keyword) use ($keyword_to_delete) {
        return strtolower($keyword) != strtolower($keyword_to_delete);
    });

    // Save the updated list of keywords
    update_post_meta($post_id, 'custom_keywords', implode(',', $updated_keywords_array));

    wp_send_json_success(array_values($updated_keywords_array)); // Return the updated list
}

?>